<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace ShipQuik\PrestaShop;

use ShipQuik\Shared\ShipQuik_API;
use ShipQuik\Shared\ShipQuik_Helper;
use ShipQuik\Shared\ShipQuik_Suppliers;

class ShipQuik_Shipments
{
    private static $_instance = null;

    protected $parent;

    protected $order_id;

    protected $boxes;

    protected $price;

    protected $save;

    protected $config;

    public $data;

    public function __construct($parent = null, $order_id = 0, $boxes = [], $save = true, $price = 0)
    {
        $this->parent = $parent;
        $this->order_id = $order_id;
        $this->boxes = $boxes;
        $this->price = $price;
        $this->save = $save;

        $this->config = $parent->config;

        $this->data = $this->getData($this->order_id, $this->boxes, $this->save, $this->price);
    }

    private function getData($order_id, $boxes, $save, $price)
    {
        $order = new \Order((int) $order_id);

        $supplierInfo = [];

        $result = ShipQuik_BD::getDatabyOrder($order);

        if (!$result) {
            return [];
        }

        $totalPayment = json_decode(str_replace('&quot;', '"', $result['total_payment']), true, 99);

        $shipment = [];

        $senderAddress = self::getSenderAddress($this->config);
        $customerAddress = ShipQuik_Suppliers::getNormalizeDestinationPrestaShop(new \Address($order->id_address_delivery));
        $customerCif = new \Address($order->id_address_delivery);
        $customerCif = ($customerCif->vat_number ? $customerCif->vat_number : $customerCif->dni); // vat_number or dni

        $id_customer = $order->id_customer;
        $customer = new \Customer((int) $id_customer);
        $customerAddress['email'] = $customer->email;

        // Shipment Object
        $shipment['shipment']['isKg'] = true;

        if (empty($boxes)) {
            $shipment['shipment']['boxes'] = ShipQuik_ShippingPackage::getBoxes($this->config, $order->getProducts());
        } else {
            $shipment['shipment']['boxes'] = $boxes;
        }

        $shipment['shipment']['originAddress'] = [
            'street' => $senderAddress['Street'],
            'city' => $senderAddress['City'],
            'countryCode' => $senderAddress['CountryCode'],
            'postalCode' => $senderAddress['PostalCode'],
            'floor' => '',
            'door' => '',
        ];

        $shipment['shipment']['destinationAddress'] = [
            'street' => $customerAddress['street'],
            'city' => $customerAddress['city'],
            'countryCode' => $customerAddress['countrycode'],
            'postalCode' => $customerAddress['postcode'],
            'floor' => '',
            'door' => '',
        ];

        $comments = $order->getFirstMessage();
        if (!$comments) {
            $comments = '';
        }

        $contact = $customerAddress['contact'];
        $pickupData = json_decode($result['pickup_datetime'], true);

        // Pickup object
        $shipment['pickup'] = [
            'name' => $this->config->getValueByKey('senderAddressCompany'),
            'cif' => $this->config->getValueByKey('senderAddressCif'),
            'phone' => [
                'countryCode' => $senderAddress['CountryCode'],
                'phoneNumber' => $this->config->getValueByKey('senderAddressPhoneNumber'),
            ],
            'email' => $this->config->getValueByKey('senderAddressEmail'),
            'comments' => $this->config->getValueByKey('comments'),
            'attention' => '',
            'date' => (isset($pickupData['pickup_date']) ? $pickupData['pickup_date'] : ''),
            'schedule' => (isset($pickupData['pickup_time']) ? $pickupData['pickup_time'] : '1'),
        ];

        $name = ($customerAddress['company'] ? $customerAddress['company'] : $customerAddress['contact']);

        // Delivery object
        $shipment['delivery'] = [
            'name' => $name,
            'cif' => $customerCif,
            'phone' => [
                'countryCode' => $customerAddress['countrycode'],
                'phoneNumber' => $customerAddress['phone'],
            ],
            'email' => $customerAddress['email'],
            'comments' => $comments,
            'attention' => $contact,
        ];

        $supplierInfo = $result['supplier_data'];

        if ($price > 0) {
            $supplierInfo['salesPrice'] = $price;
        }

        // Supplier object
        $shipment['supplier'] = $supplierInfo;
        $shipment['supplier']['currency'] = '';
        $shipment['supplier']['serviceCode'] = (isset($supplierInfo['serviceCode']) ? $supplierInfo['serviceCode'] : '');

        $shipment['dua'] = [
            'duaSale' => $supplierInfo['dua'],
        ];

        $taricLines = [];
        if ($supplierInfo['isTaric']) {
            foreach ($shipment['shipment']['boxes'] as $_box) {
                $taricLines[] = [
                    'quantity' => 1,
                    'description' => 'Cajas, bolsas de papel | 358',
                    'value' => 10,
                ];
            }
        }
        $shipment['insurance'] = [
            'insurancePrice' => 0,
            'declaredValue' => 0,
            'insuranceCost' => 0,
            'declaredObjects' => 'Otros',
            'contentAgreement' => true,
            'isInternational' => $supplierInfo['isTaric'],
            'isTaric' => $supplierInfo['isTaric'],
            'tarics' => $taricLines,
        ];

        $shipment['iva'] = [
            'ivaPercentage' => 0,
            'ivaVentaName' => 'NOSUJETO',
            'ivaCompraName' => 'IGIAND45',
            'operationVName' => 'VNAC',
            'operationCName' => 'SUJPAS',
        ];

        $shipment['payment'] = [
            'total' => $supplierInfo['salesPrice'] + $supplierInfo['dua'],
            'sale' => $supplierInfo['salesPrice'],
            'method' => (string) $this->config->getPaymentMethod(),
        ];

        $uuid = ShipQuik_Helper::getUUID($supplierInfo['supplier']);

        $shipment['seriesInfo'] = [
            'salesOrder' => 'PV-PS',
            'purchaseOrder' => 'PC-PS',
            'invoice' => 'FV-PS',
        ];

        $shipment['additionalProperties'] = [
            'sendEmails' => false,
        ];

        if (isset($totalPayment['reference'])) {
            $shipment['orderId'] = $totalPayment['reference'];
            $shipment['reference'] = $totalPayment['reference'];
        } else {
            $shipment['orderId'] = $uuid;
            $shipment['reference'] = $uuid;
        }

        $shipment['userEmail'] = $this->config->getValueByKey('senderAddressEmail');
        $shipment['clientId'] = $this->config->getIdCliente();

        if ($this->save) {
            ShipQuik_BD::setTotalPayment($order, json_encode($shipment, true));
        }

        // echo json_encode($shipment, true);
        // die();

        return $shipment;
    }

    public static function getSenderAddress($config)
    {
        $senderAddress = [];

        $senderAddress['Street'] = $config->getValueByKey('senderAddressStreet');
        $senderAddress['City'] = $config->getValueByKey('senderAddressCity');
        $senderAddress['CountryCode'] = $config->getValueByKey('senderAddressCountryCode');
        $senderAddress['PostalCode'] = $config->getValueByKey('senderAddressPostalCode');
        $senderAddress['Country'] = 'España';

        return $senderAddress;
    }

    public function send()
    {
        $params = [];
        $params['Baseurl'] = $this->config->getValueByKey('endpoints')['createShipment'];
        $params['Data'] = $this->data;

        // echo json_encode($params['Data']); die();

        $curl = new ShipQuik_API($this->parent, $params, true);
        $result = $curl->runCall();

        if ($result == 'renew_token') {
            $this->config->reNewAccessToken();

            $result = $curl->runCall();
        }

        return $result;
    }

    public static function instance()
    {
        if (!isset(self::$_instance)) {
            $class = __CLASS__;
            self::$_instance = new $class();
        }

        return self::$_instance;
    }

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
