<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace ShipQuik\PrestaShop;

class ShipQuik_ShippingPackage
{
    protected $parent;

    protected $config;

    protected $packages;

    protected $measureUnit;

    public function __construct($parent, $packages, $measureUnit = 'kg')
    {
        $this->parent = $parent;
        $this->config = $parent->config;

        $this->packages = $packages;
        $this->measureUnit = $measureUnit;
    }

    /**
     * Calculates the cubage of all products.
     *
     * @return float|int
     */
    public function getTotalCubage()
    {
        $total = 0;

        foreach ($this->packages as $values) {
            $_product = $values['data'];
            $_product_height = (float) $_product->get_height();
            $_product_width = (float) $_product->get_width();
            $_product_length = (float) $_product->get_length();

            if ($_product_height == 0) {
                $_product_height = $this->config->getValueByKey('defaultParcelHeight');
            }
            if ($_product_width == 0) {
                $_product_width = $this->config->getValueByKey('defaultParcelWidth');
            }
            if ($_product_length == 0) {
                $_product_length = $this->config->getValueByKey('defaultParcelLength');
            }

            $total += $_product_height * $_product_width * $_product_length;
        }

        return number_format(($total > 0) ? ($total / 1000000) : 0, 2);
    }

    public static function getDefaultPackage()
    {
        $packagesInfo = unserialize(\Configuration::get('_SHIP_QUIK_shippingPackageList'));

        foreach ($packagesInfo as $package) {
            if (isset($package['default'])) {
                return $package;
            }
        }

        return false;
    }

    /**
     * Get the boxes of all products.
     *
     * @return array
     */
    public static function getBoxes($config, $productItems)
    {
        $boxes = [];

        $defaultPackage = self::getDefaultPackage();

        foreach ($productItems as $_product) {
            $quantity = (isset($_product['product_quantity']) ? $_product['product_quantity'] : $_product['quantity']);

            for ($i = 1; $i <= $quantity; ++$i) {
                if ((float) $_product['depth'] == 0 || (float) $_product['width'] == 0 || (float) $_product['height'] == 0 || (float) $_product['weight'] == 0) {
                    $boxes[] = [
                        'weight' => (string) $defaultPackage['weight'],
                        'length' => (string) (float) $defaultPackage['depth'],
                        'width' => (string) (float) $defaultPackage['width'],
                        'height' => (string) (float) $defaultPackage['height'],
                        'products' => [
                            'name' => isset($_product['product_name']) ? str_replace("'", '', $_product['product_name']) :
                            str_replace("'", '', $_product['name']) . ($_product['attributes'] ? ' - ' . str_replace("'", '', $_product['attributes']) : ''),
                            'qty' => 1,
                        ],
                    ];
                } else {
                    $boxes[] = [
                        'weight' => (string) $_product['weight'],
                        'length' => (string) (float) $_product['depth'],
                        'width' => (string) (float) $_product['width'],
                        'height' => (string) (float) $_product['height'],
                        'products' => [
                            'name' => isset($_product['product_name']) ? str_replace("'", '', $_product['product_name']) :
                            str_replace("'", '', $_product['name']) . ($_product['attributes'] ? ' - ' . str_replace("'", '', $_product['attributes']) : ''),
                            'qty' => 1,
                        ],
                    ];
                }
            }
        }

        return $boxes;
    }

    /**
     * Calculate the total volumes of all products of the order
     *
     * @return int
     */
    public function getTotalVolumes()
    {
        $total = 0;

        foreach ($this->packages['contents'] as $item_id => $product) {
            $total += (int) $product['cart_quantity'];
        }

        return $total;
    }

    /*
     * Calculate the total wheight of all products of the order
     *
     * @return float|int
     */
    // public function getOrderTotalWeight()
    // {
    //     $total = 0;

    //     foreach ($this->packages['contents'] as $item_id => $product) {
    //         $_product_weight = (float) $_product['weight'];

    //         if ($_product_weight == 0) {
    //             $_product_weight = $this->config->getValueByKey('defaultParcelWeight');
    //         }

    //         $total += $_product_weight * $product['cart_quantity'];
    //     }

    //     $total = wc_get_weight($total, $this->measureUnit);

    //     return $total;
    // }
}
