<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace ShipQuik\Shared;

use ShipQuik\PrestaShop\ShipQuik_BD;
use ShipQuik\PrestaShop\ShipQuik_Shipments;

class ShipQuik_Suppliers
{
    private static $_instance = null;

    protected $parent;

    protected $boxes;

    protected $destination;

    protected $id_carrier;

    protected $config;

    protected $data;

    public function __construct($parent = null, $boxes = [], $destination = [], $id_carrier = 0)
    {
        $this->parent = $parent;
        $this->boxes = $boxes;
        $this->destination = $destination;

        $this->config = $parent->config;

        $this->data = $this->getData($this->boxes, $this->destination);
        $this->id_carrier = $id_carrier;
    }

    public static function getNormalizeDestinationPrestaShop($destination)
    {
        $resultAddress = [];

        // package destination
        $country = new \Country($destination->id_country);
        $resultAddress['countrycode'] = $country->iso_code;

        if (isset($destination->country)) {
            $resultAddress['country'] = $destination->country;
        }
        if (isset($destination->postcode)) {
            $resultAddress['postcode'] = $destination->postcode;
        }
        if (isset($destination->city)) {
            $resultAddress['city'] = $destination->city;
        }
        if (isset($destination->address1)) {
            $resultAddress['street'] = $destination->address1 . ' ' . $destination->address2;
        }
        if (isset($destination->phone)) {
            $resultAddress['phone'] = $destination->phone;
        }
        if (isset($destination->company)) {
            $resultAddress['company'] = $destination->company;
        }
        if (isset($destination->firstname) && isset($destination->lastname)) {
            $resultAddress['contact'] = $destination->firstname . ' ' . $destination->firstname;
        }

        $state = new \State($destination->id_state);
        $resultAddress['state'] = $state->name;

        return $resultAddress;
    }

    private function getData($boxes, $destination)
    {
        $supplierData = [];

        $customerAddress = self::getNormalizeDestinationPrestaShop($destination);
        $senderAddress = ShipQuik_Shipments::getSenderAddress($this->config);

        $supplierData['onlyPrice'] = true;

        $supplierData['postalCodeOrigin'] = $senderAddress['PostalCode'];
        $supplierData['postalCodeDestination'] = $customerAddress['postcode'];
        $supplierData['countryCodeOrigin'] = $senderAddress['CountryCode'];
        $supplierData['countryCodeDestination'] = $customerAddress['countrycode'];

        $supplierData['pesoEnKG'] = true;

        $supplierData['boxes'] = $boxes;

        $supplierData['uID'] = (string) $this->config->getIdCliente();

        return $supplierData;
    }

    public function send($suppliers, $searchMode)
    {
        $result = false;

        $params = [];

        if ($searchMode === 'speed') {
            $searchMode = 'lowest-speed';
        }
        if ($searchMode === 'price') {
            $searchMode = 'lowest-price';
        }
        if ($searchMode === 'speedprice') {
            $searchMode = 'lowest-both';
        }

        $params['Baseurl'] = $this->config->getValueByKey('endpoints')['getSupplierBaseUri'];
        $params['Data'] = $this->data;
        $params['Data']['suppliers'] = $suppliers;
        $params['Data']['searchMode'] = $searchMode;
        $params['Data']['pickupInfo'] = true;

        // d(json_encode($params['Data']));

        $context = \Context::getContext();
        $cache = ShipQuik_BD::getCacheData('cache_data_suppliers', $context->cart, $this->id_carrier);

        if ($cache) {
            if (serialize($cache['params']) === serialize($params)) {
                return $cache['result'];
            }
        }

        $curl = new ShipQuik_API($this->parent, $params, true);
        $result = $curl->runCall();

        if ($result == 'renew_token') {
            $this->config->reNewAccessToken();
            $result = $curl->runCall();
        }

        $resultArray = [];

        if ($result) {
            foreach ($result as $element) {
                unset($element['serviceObservations']);
                $resultArray[] = $element;
            }
        }

        $data = ['params' => $params, 'result' => $resultArray];

        if ($result) {
            $sql = ShipQuik_BD::setCacheData('cache_data_suppliers', $context->cart, $this->id_carrier, $data);
        }

        return $resultArray;
    }

    public function getSuppliersResult($suppliers = [], $pricingMode)
    {
        if (empty($suppliers)) {
            $suppliers = $this->config->getValueByKey('suppliers');
        }

        $suppliersResult = $this->send(unserialize($suppliers), $pricingMode);

        return $suppliersResult;
    }

    public function getServiceByName($shipQuikSuppliers, $name)
    {
        foreach ($shipQuikSuppliers as $service) {
            if ($service['service'] == $name) {
                return $service;
            }
        }

        return false;
    }

    public function getServiceByType($shipQuikSuppliers, $type)
    {
        foreach ($shipQuikSuppliers as $service) {
            if ($service['type'] == $type) {
                return $service;
            }
        }

        return false;
    }

    public static function instance()
    {
        if (!isset(self::$_instance)) {
            $class = __CLASS__;
            self::$_instance = new $class();
        }

        return self::$_instance;
    }

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
